@@ -1,6 +1,6 @@
 /*
    neon SSL/TLS support using GNU TLS
-   Copyright (C) 2002-2008, Joe Orton <joe@manyfish.co.uk>
+   Copyright (C) 2002-2006, Joe Orton <joe@manyfish.co.uk>
    Copyright (C) 2004, Aleix Conchillo Flaque <aleix@member.fsf.org>
 
    This library is free software; you can redistribute it and/or
@@ -17,10 +17,13 @@
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
    MA 02111-1307, USA
+
 */
 
 #include "config.h"
 
+#include <sys/mman.h>
+#include <sys/stat.h>
 #include <sys/types.h>
 
 #ifdef HAVE_STRING_H
@@ -30,7 +33,6 @@
 #include <stdio.h>
 #include <unistd.h>
 #include <stdlib.h>
-#include <errno.h>
 
 #include <gnutls/gnutls.h>
 #include <gnutls/pkcs12.h>
@@ -40,12 +42,6 @@
 #include <pthread.h>
 #include <gcrypt.h>
 GCRY_THREAD_OPTION_PTHREAD_IMPL;
-#else
-#include <gcrypt.h>
-#endif
-
-#ifdef HAVE_ICONV
-#include <iconv.h>
 #endif
 
 #include "ne_ssl.h"
@@ -71,7 +67,6 @@ struct ne_ssl_certificate_s {
 struct ne_ssl_client_cert_s {
     gnutls_pkcs12 p12;
     int decrypted; /* non-zero if successfully decrypted. */
-    int keyless;
     ne_ssl_certificate cert;
     gnutls_x509_privkey pkey;
     char *friendly_name;
@@ -98,132 +93,6 @@ static int oid_find_highest_index(gnutls_x509_crt cert, int subject, const char
     return idx - 1;
 }
 
-#ifdef HAVE_GNUTLS_X509_DN_GET_RDN_AVA
-/* New-style RDN handling introduced in GnuTLS 1.7.x. */
-
-#ifdef HAVE_ICONV
-static void convert_dirstring(ne_buffer *buf, const char *charset, 
-                              gnutls_datum *data)
-{
-    iconv_t id = iconv_open("UTF-8", charset);
-    size_t inlen = data->size, outlen = buf->length - buf->used;
-    char *inbuf = (char *)data->data;
-    char *outbuf = buf->data + buf->used - 1;
-    
-    if (id == (iconv_t)-1) {
-        char err[128], err2[128];
-
-        ne_snprintf(err, sizeof err, "[unprintable in %s: %s]",
-                    charset, ne_strerror(errno, err2, sizeof err2));
-        ne_buffer_zappend(buf, err);
-        return;
-    }
-    
-    ne_buffer_grow(buf, buf->used + 64);
-    
-    while (inlen && outlen 
-           && iconv(id, &inbuf, &inlen, &outbuf, &outlen) == 0)
-        ;
-    
-    iconv_close(id);
-    buf->used += buf->length - buf->used - outlen;
-    buf->data[buf->used - 1] = '\0';
-}
-#endif
-
-/* From section 11.13 of the Dubuisson ASN.1 bible: */
-#define TAG_UTF8 (12)
-#define TAG_PRINTABLE (19)
-#define TAG_T61 (20)
-#define TAG_IA5 (22)
-#define TAG_VISIBLE (26)
-#define TAG_UNIVERSAL (28)
-#define TAG_BMP (30)
-
-static void append_dirstring(ne_buffer *buf, gnutls_datum *data, unsigned long tag)
-{
-    switch (tag) {
-    case TAG_UTF8:
-    case TAG_IA5:
-    case TAG_PRINTABLE:
-    case TAG_VISIBLE:
-        ne_buffer_append(buf, (char *)data->data, data->size);
-        break;
-#ifdef HAVE_ICONV
-    case TAG_T61:
-        convert_dirstring(buf, "ISO-8859-1", data);
-        break;
-    case TAG_BMP:
-        convert_dirstring(buf, "UCS-2BE", data);
-        break;
-#endif
-    default: {
-        char tmp[128];
-        ne_snprintf(tmp, sizeof tmp, _("[unprintable:#%lu]"), tag);
-        ne_buffer_zappend(buf, tmp);
-    } break;
-    }
-}
-
-/* OIDs to not include in readable DNs by default: */
-#define OID_emailAddress "1.2.840.113549.1.9.1"
-#define OID_commonName "2.5.4.3"
-
-#define CMPOID(a,o) ((a)->oid.size == sizeof(o)                        \
-                     && memcmp((a)->oid.data, o, strlen(o)) == 0)
-
-char *ne_ssl_readable_dname(const ne_ssl_dname *name)
-{
-    gnutls_x509_dn_t dn;
-    int ret, rdn = 0, flag = 0;
-    ne_buffer *buf;
-    gnutls_x509_ava_st val;
-
-    if (name->subject)
-        ret = gnutls_x509_crt_get_subject(name->cert, &dn);
-    else
-        ret = gnutls_x509_crt_get_issuer(name->cert, &dn);
-    
-    if (ret)
-        return ne_strdup(_("[unprintable]"));
-
-    buf = ne_buffer_create();
-    
-    /* Find the highest rdn... */
-    while (gnutls_x509_dn_get_rdn_ava(dn, rdn++, 0, &val) == 0)
-        ;        
-
-    /* ..then iterate back to the first: */
-    while (--rdn >= 0) {
-        int ava = 0;
-
-        /* Iterate through all AVAs for multivalued AVAs; better than
-         * ne_openssl can do! */
-        do {
-            ret = gnutls_x509_dn_get_rdn_ava(dn, rdn, ava, &val);
-
-            /* If the *only* attribute to append is the common name or
-             * email address, use it; otherwise skip those
-             * attributes. */
-            if (ret == 0 && val.value.size > 0
-                && ((!CMPOID(&val, OID_emailAddress)
-                     && !CMPOID(&val, OID_commonName))
-                    || (buf->used == 1 && rdn == 0))) {
-                flag = 1;
-                if (buf->used > 1) ne_buffer_append(buf, ", ", 2);
-
-                append_dirstring(buf, &val.value, val.value_tag);
-            }
-            
-            ava++;
-        } while (ret == 0);
-    }
-
-    return ne_buffer_finish(buf);
-}
-
-#else /* !HAVE_GNUTLS_X509_DN_GET_RDN_AVA */
-
 /* Appends the value of RDN with given oid from certitifcate x5
  * subject (if subject is non-zero), or issuer DN to buffer 'buf': */
 static void append_rdn(ne_buffer *buf, gnutls_x509_crt x5, int subject, const char *oid)
@@ -273,7 +142,6 @@ char *ne_ssl_readable_dname(const ne_ssl_dname *name)
 
     return ne_buffer_finish(buf);
 }
-#endif /* HAVE_GNUTLS_X509_DN_GET_RDN_AVA */
 
 int ne_ssl_dname_cmp(const ne_ssl_dname *dn1, const ne_ssl_dname *dn2)
 {
@@ -325,12 +193,34 @@ void ne_ssl_cert_validity_time(const ne_ssl_certificate *cert,
     }
 }
 
+/* Return non-zero if hostname from certificate (cn) matches hostname
+ * used for session (hostname).  (Wildcard matching is no longer
+ * mandated by RFC3280, but certs are deployed which use wildcards) */
+static int match_hostname(char *cn, const char *hostname)
+{
+    const char *dot;
+    NE_DEBUG(NE_DBG_SSL, "Match %s on %s...\n", cn, hostname);
+    dot = strchr(hostname, '.');
+    if (dot == NULL) {
+	char *pnt = strchr(cn, '.');
+	/* hostname is not fully-qualified; unqualify the cn. */
+	if (pnt != NULL) {
+	    *pnt = '\0';
+	}
+    }
+    else if (strncmp(cn, "*.", 2) == 0) {
+	hostname = dot + 1;
+	cn += 2;
+    }
+    return !ne_strcasecmp(cn, hostname);
+}
+
 /* Check certificate identity.  Returns zero if identity matches; 1 if
  * identity does not match, or <0 if the certificate had no identity.
  * If 'identity' is non-NULL, store the malloc-allocated identity in
  * *identity.  If 'server' is non-NULL, it must be the network address
  * of the server in use, and identity must be NULL. */
-static int check_identity(const ne_uri *server, gnutls_x509_crt cert,
+static int check_identity(const char *hostname, gnutls_x509_crt cert,
                           char **identity)
 {
     char name[255];
@@ -338,19 +228,15 @@ static int check_identity(const ne_uri *server, gnutls_x509_crt cert,
     int ret, seq = 0;
     int match = 0, found = 0;
     size_t len;
-    const char *hostname;
-    
-    hostname = server ? server->host : "";
 
     do {
-        len = sizeof name - 1;
+        len = sizeof name;
         ret = gnutls_x509_crt_get_subject_alt_name(cert, seq, name, &len,
                                                    &critical);
         switch (ret) {
         case GNUTLS_SAN_DNSNAME:
-            name[len] = '\0';
             if (identity && !found) *identity = ne_strdup(name);
-            match = ne__ssl_match_hostname(name, len, hostname);
+            match = match_hostname(name, hostname);
             found = 1;
             break;
         case GNUTLS_SAN_IPADDRESS: {
@@ -375,32 +261,6 @@ static int check_identity(const ne_uri *server, gnutls_x509_crt cert,
                          len);
             }
         } break;
-        case GNUTLS_SAN_URI: {
-            ne_uri uri;
-            
-            name[len] = '\0';
-            
-            if (ne_uri_parse(name, &uri) == 0 && uri.host && uri.scheme) {
-                ne_uri tmp;
-                
-                if (identity && !found) *identity = ne_strdup(name);
-                found = 1;
-                
-                if (server) {
-                    /* For comparison purposes, all that matters is
-                     * host, scheme and port; ignore the rest. */
-                    memset(&tmp, 0, sizeof tmp);
-                    tmp.host = uri.host;
-                    tmp.scheme = uri.scheme;
-                    tmp.port = uri.port;
-                    
-                    match = ne_uri_cmp(server, &tmp) == 0;
-                }
-            }
-            
-            ne_uri_free(&uri);
-        } break;
-
         default:
             break;
         }
@@ -419,17 +279,14 @@ static int check_identity(const ne_uri *server, gnutls_x509_crt cert,
                                                 seq, 0, name, &len);
             if (ret == 0) {
                 if (identity) *identity = ne_strdup(name);
-                match = ne__ssl_match_hostname(name, len, hostname);
+                match = match_hostname(name, hostname);
             }
         } else {
             return -1;
         }
     }
 
-    if (*hostname)
-        NE_DEBUG(NE_DBG_SSL, "ssl: Identity match for '%s': %s\n", hostname, 
-                 match ? "good" : "bad");
-
+    NE_DEBUG(NE_DBG_SSL, "Identity match: %s\n", match ? "good" : "bad");
     return match ? 0 : 1;
 }
 
@@ -444,7 +301,7 @@ static ne_ssl_certificate *populate_cert(ne_ssl_certificate *cert,
     cert->issuer = NULL;
     cert->subject = x5;
     cert->identity = NULL;
-    check_identity(NULL, x5, &cert->identity);
+    check_identity("", x5, &cert->identity);
     return cert;
 }
 
@@ -489,18 +346,13 @@ static ne_ssl_client_cert *dup_client_cert(const ne_ssl_client_cert *cc)
     ne_ssl_client_cert *newcc = ne_calloc(sizeof *newcc);
 
     newcc->decrypted = 1;
-    
-    if (cc->keyless) {
-        newcc->keyless = 1;
-    }
-    else {
-        ret = gnutls_x509_privkey_init(&newcc->pkey);
-        if (ret != 0) goto dup_error;
-        
-        ret = gnutls_x509_privkey_cpy(newcc->pkey, cc->pkey);
-        if (ret != 0) goto dup_error;
-    }    
 
+    ret = gnutls_x509_privkey_init(&newcc->pkey);
+    if (ret != 0) goto dup_error;
+
+    ret = gnutls_x509_privkey_cpy(newcc->pkey, cc->pkey);
+    if (ret != 0) goto dup_error;
+    
     newcc->cert.subject = x509_crt_copy(cc->cert.subject);
     if (!newcc->cert.subject) goto dup_error;
 
@@ -525,7 +377,7 @@ static int provide_client_cert(gnutls_session session,
     ne_session *sess = gnutls_session_get_ptr(session);
     
     if (!sess) {
-        return GNUTLS_E_RECEIVED_ILLEGAL_PARAMETER;
+        return -1;
     }
 
     if (!sess->client_cert && sess->ssl_provide_fn) {
@@ -550,12 +402,10 @@ static int provide_client_cert(gnutls_session session,
             /* tell GNU TLS not to deallocate the certs. */
             st->deinit_all = 0;
         } else {
-            return GNUTLS_E_UNSUPPORTED_CERTIFICATE_TYPE;
+            return -1;
         }
     } else {
         NE_DEBUG(NE_DBG_SSL, "No client certificate supplied.\n");
-        sess->ssl_cc_requested = 1;
-        return GNUTLS_E_NO_CERTIFICATE_FOUND;
     }
 
     return 0;
@@ -574,8 +424,6 @@ ne_ssl_context *ne_ssl_context_create(int flags)
         gnutls_certificate_client_set_retrieve_function(ctx->cred,
                                                         provide_client_cert);
     }
-    gnutls_certificate_set_verify_flags(ctx->cred, 
-                                        GNUTLS_VERIFY_ALLOW_X509_V1_CA_CRT);
     return ctx;
 }
 
@@ -636,9 +484,7 @@ static ne_ssl_certificate *make_peers_chain(gnutls_session sock)
 
         if (gnutls_x509_crt_init(&x5) ||
             gnutls_x509_crt_import(x5, &certs[n], GNUTLS_X509_FMT_DER)) {
-            if (top) {
-                ne_ssl_cert_free(top);
-            }
+            ne_ssl_cert_free(top);
             return NULL;
         }
 
@@ -661,7 +507,6 @@ static int check_certificate(ne_session *sess, gnutls_session sock,
 {
     time_t before, after, now = time(NULL);
     int ret, failures = 0;
-    ne_uri server;
 
     before = gnutls_x509_crt_get_activation_time(chain->subject);
     after = gnutls_x509_crt_get_expiration_time(chain->subject);
@@ -671,11 +516,7 @@ static int check_certificate(ne_session *sess, gnutls_session sock,
     else if (now > after)
         failures |= NE_SSL_EXPIRED;
 
-    memset(&server, 0, sizeof server);
-    ne_fill_server_uri(sess, &server);
-    ret = check_identity(&server, chain->subject, NULL);
-    ne_uri_free(&server);
-
+    ret = check_identity(sess->server.hostname, chain->subject, NULL);
     if (ret < 0) {
         ne_set_error(sess, _("Server certificate was missing commonName "
                              "attribute in subject name"));
@@ -712,21 +553,10 @@ int ne__negotiate_ssl(ne_session *sess)
 
     NE_DEBUG(NE_DBG_SSL, "Negotiating SSL connection.\n");
 
-    /* Pass through the hostname if SNI is enabled. */
-    ctx->hostname = 
-        sess->flags[NE_SESSFLAG_TLS_SNI] ? sess->server.hostname : NULL;
-
     if (ne_sock_connect_ssl(sess->socket, ctx, sess)) {
-        if (sess->ssl_cc_requested) {
-            ne_set_error(sess, _("SSL negotiation failed, "
-                                 "client certificate was requested: %s"),
-                         ne_sock_error(sess->socket));
-        }
-        else {
-            ne_set_error(sess, _("SSL negotiation failed: %s"),
-                         ne_sock_error(sess->socket));
-        }
-        return NE_ERROR;
+	ne_set_error(sess, _("SSL negotiation failed: %s"),
+		     ne_sock_error(sess->socket));
+	return NE_ERROR;
     }
 
     sock = ne__sock_sslsock(sess->socket);
@@ -808,13 +638,10 @@ static int read_to_datum(const char *filename, gnutls_datum *datum)
     }
 
     if (!feof(f)) {
-        fclose(f);
         ne_buffer_destroy(buf);
         return -1;
     }
     
-    fclose(f);
-
     datum->size = ne_buffer_size(buf);
     datum->data = (unsigned char *)ne_buffer_finish(buf);
     return 0;
@@ -839,14 +666,14 @@ static int pkcs12_parse(gnutls_pkcs12 p12, gnutls_x509_privkey *pkey,
         ret = gnutls_pkcs12_get_bag(p12, i, bag);
         if (ret < 0) continue;
 
-        gnutls_pkcs12_bag_decrypt(bag, password);
+        gnutls_pkcs12_bag_decrypt(bag, password == NULL ? "" : password);
 
         for (j = 0; ret == 0 && j < gnutls_pkcs12_bag_get_count(bag); ++j) {
             gnutls_pkcs12_bag_type type;
             gnutls_datum data;
 
             if (friendly_name && *friendly_name == NULL) {
-                char *name = NULL;
+                char *name;
                 gnutls_pkcs12_bag_get_friendly_name(bag, j, &name);
                 if (name) {
                     if (name[0] == '.') name++; /* weird GnuTLS bug? */
@@ -858,10 +685,6 @@ static int pkcs12_parse(gnutls_pkcs12 p12, gnutls_x509_privkey *pkey,
             switch (type) {
             case GNUTLS_BAG_PKCS8_KEY:
             case GNUTLS_BAG_PKCS8_ENCRYPTED_KEY:
-                /* Ignore any but the first key encountered; really
-                 * need to match up keyids. */
-                if (*pkey) break;
-
                 gnutls_x509_privkey_init(pkey);
 
                 ret = gnutls_pkcs12_bag_get_data(bag, j, &data);
@@ -874,10 +697,6 @@ static int pkcs12_parse(gnutls_pkcs12 p12, gnutls_x509_privkey *pkey,
                 if (ret < 0) continue;
                 break;
             case GNUTLS_BAG_CERTIFICATE:
-                /* Ignore any but the first cert encountered; again,
-                 * really need to match up keyids. */
-                if (*x5) break;
-
                 gnutls_x509_crt_init(x5);
 
                 ret = gnutls_pkcs12_bag_get_data(bag, j, &data);
@@ -931,13 +750,9 @@ ne_ssl_client_cert *ne_ssl_clicert_read(const char *filename)
         return NULL;
     }
 
-    if (gnutls_pkcs12_verify_mac(p12, "") == 0) {
-        if (pkcs12_parse(p12, &pkey, &cert, &friendly_name, "") != 0
-            || !cert || !pkey) {
-            gnutls_pkcs12_deinit(p12);
-            return NULL;
-        }
+    ret = pkcs12_parse(p12, &pkey, &cert, &friendly_name, NULL);
 
+    if (gnutls_pkcs12_verify_mac(p12, "") == 0) {
         cc = ne_calloc(sizeof *cc);
         cc->pkey = pkey;
         cc->decrypted = 1;
@@ -947,38 +762,18 @@ ne_ssl_client_cert *ne_ssl_clicert_read(const char *filename)
         cc->p12 = NULL;
         return cc;
     } else {
-        /* TODO: calling pkcs12_parse() here to find the friendly_name
-         * seems to break horribly.  */
-        cc = ne_calloc(sizeof *cc);
-        cc->p12 = p12;
-        return cc;
+        if (ret == 0) {
+            cc = ne_calloc(sizeof *cc);
+            cc->friendly_name = friendly_name;
+            cc->p12 = p12;
+            return cc;
+        } else {
+            gnutls_pkcs12_deinit(p12);
+            return NULL;
+        }
     }
 }
 
-ne_ssl_client_cert *ne__ssl_clicert_exkey_import(const unsigned char *der,
-                                                 size_t der_len)
-{
-    ne_ssl_client_cert *cc;
-    gnutls_x509_crt x5;
-    gnutls_datum datum;
-
-    datum.data = (unsigned char *)der;
-    datum.size = der_len;    
-
-    if (gnutls_x509_crt_init(&x5) 
-        || gnutls_x509_crt_import(x5, &datum, GNUTLS_X509_FMT_DER)) {
-        NE_DEBUG(NE_DBG_SSL, "ssl: crt_import failed.\n");
-        return NULL;
-    }
-    
-    cc = ne_calloc(sizeof *cc);
-    cc->keyless = 1;
-    cc->decrypted = 1;
-    populate_cert(&cc->cert, x5);
-
-    return cc;    
-}
-
 int ne_ssl_clicert_encrypted(const ne_ssl_client_cert *cc)
 {
     return !cc->decrypted;
@@ -997,12 +792,6 @@ int ne_ssl_clicert_decrypt(ne_ssl_client_cert *cc, const char *password)
     ret = pkcs12_parse(cc->p12, &pkey, &cert, NULL, password);
     if (ret < 0)
         return ret;
-    
-    if (!cert || (!pkey && !cc->keyless)) {
-        if (cert) gnutls_x509_crt_deinit(cert);
-        if (pkey) gnutls_x509_privkey_deinit(pkey);
-        return -1;
-    }
 
     gnutls_pkcs12_deinit(cc->p12);
     populate_cert(&cc->cert, cert);
@@ -1169,17 +958,15 @@ int ne__ssl_init(void)
 #ifdef NE_HAVE_TS_SSL
     gcry_control(GCRYCTL_SET_THREAD_CBS, &gcry_threads_pthread);
 #endif
-    gcry_control(GCRYCTL_ENABLE_QUICK_RANDOM, 0);
     return gnutls_global_init();
 }
 
 void ne__ssl_exit(void)
 {
     /* No way to unregister the thread callbacks.  Doomed. */
-#if LIBGNUTLS_VERSION_MAJOR > 1 || LIBGNUTLS_VERSION_MINOR > 3 \
-    || (LIBGNUTLS_VERSION_MINOR == 3 && LIBGNUTLS_VERSION_PATCH >= 3)
+#if 0
     /* It's safe to call gnutls_global_deinit() here only with
-     * gnutls >= 1.3., since older versions don't refcount and
+     * gnutls >= 1.3, since older versions don't refcount and
      * doing so would prevent any other use of gnutls within
      * the process. */
     gnutls_global_deinit();
