@@ -66,39 +66,28 @@ static int no_path(void)
 
 static int escapes(void)
 {
-    static const struct {
+    const struct {
         const char *plain, *escaped;
-        unsigned int flags;
     } paths[] = {
-        { "/foo%", "/foo%25", 0 },
-        { "/foo bar", "/foo%20bar", 0, },
-        { "/foo_bar", "/foo_bar", 0 },
-        { "/foobar", "/foobar", 0 },
-        { "/a\xb9\xb2\xb3\xbc\xbd/", "/a%b9%b2%b3%bc%bd/", 0 },
-        { "/foo%20\xb9\xb2\xb3\xbc\xbd/", "/foo%20%b9%b2%b3%bc%bd/", NE_PATH_NONURI },
-        { "/foo bar/", "/foo%20bar/", NE_PATH_NONURI },
+        { "/foo%", "/foo%25" },
+        { "/foo bar", "/foo%20bar" },
+        { "/foo_bar", "/foo_bar" },
+        { "/foobar", "/foobar" },
+        { "/a\xb9\xb2\xb3\xbc\xbd/", "/a%b9%b2%b3%bc%bd/" },
         { NULL, NULL}
     };
     size_t n;
 
     for (n = 0; paths[n].plain; n++) {
-        char *esc;
-
-        if (paths[n].flags)
-            esc = ne_path_escapef(paths[n].plain, paths[n].flags);
-        else
-            esc = ne_path_escape(paths[n].plain);
+        char *esc = ne_path_escape(paths[n].plain), *un;
 
         ONCMP(paths[n].escaped, esc, paths[n].plain, "escape");
+        
+        un = ne_path_unescape(esc);
 
-        if (!paths[n].flags) {
-            char *un = ne_path_unescape(esc);
-
-            ONCMP(paths[n].plain, un, paths[n].plain, "unescape");
-            ne_free(un);
-        }
-
-        ne_free(esc);
+        ONCMP(paths[n].plain, un, paths[n].plain, "unescape");
+        ne_free(un);
+        ne_free(esc);   
     }
 
     ONN("unescape accepted invalid URI", 
@@ -233,7 +222,7 @@ static int cmp(void)
             ("'%s' and '%s' did not compare as different",
              diff[n].left, diff[n].right));
 
-        ONV(((r1 > 0) != (r2 < 0) || (r1 < 0) != (r2 > 0)),
+        ONV(r1 + r2 != 0,
             ("'%s' and '%s' did not compare reflexively (%d vs %d)",
              diff[n].left, diff[n].right, r1, r2));
 
@@ -296,8 +285,6 @@ static int parse(void)
         { "http://foo/bar#beta", "http", "foo", 0, "/bar", NULL, NULL, "beta" },
         { "http://foo/bar?#beta", "http", "foo", 0, "/bar", NULL, "", "beta" },
         { "http://foo/bar?alpha?beta", "http", "foo", 0, "/bar", NULL, "alpha?beta", NULL },
-        { "http://foo?alpha", "http", "foo", 0, "/", NULL, "alpha", NULL },
-        { "http://foo#beta", "http", "foo", 0, "/", NULL, NULL, "beta" },
 
         /* Examples from RFC39861.1.2: */
         { "ftp://ftp.is.co.za/rfc/rfc1808.txt", "ftp", "ftp.is.co.za", 0, "/rfc/rfc1808.txt", NULL, NULL, NULL },
@@ -321,10 +308,6 @@ static int parse(void)
         { "http://[::1]:8080/bar", "http", "[::1]", 8080, "/bar", NULL, NULL, NULL },
         { "ftp://[feed::cafe]:555", "ftp", "[feed::cafe]", 555, "/", NULL, NULL, NULL },
 
-        /* Test RFC 6874 syntax, an extension of RFC 3987. */
-        { "http://[fe80::cafe%25eth0]:555", "http", "[fe80::cafe%25eth0]", 555, "/", NULL, NULL, NULL },
-        { "http://[fe80::cafe%251]:555", "http", "[fe80::cafe%251]", 555, "/", NULL, NULL, NULL },
-
         { "DAV:", "DAV", NULL, 0, "", NULL, NULL, NULL },
         
         /* Some odd cases: heir-part and relative-ref will both match
@@ -347,9 +330,6 @@ static int parse(void)
         { "foo/bar", NULL, NULL, 0, "foo/bar", NULL, NULL, NULL }, /* path-noscheme */
         { "", NULL, NULL, 0, "", NULL, NULL, NULL }, /* path-empty */
 
-        /* CVE-2007-0157: buffer under-read in 0.26.[012]. */
-        { "http://webdav.org\xFF", "http", "webdav.org\xFF", 0, "/", NULL, NULL, NULL },
-
 	{ NULL }
     };
     int n;
@@ -377,14 +357,12 @@ static int parse(void)
 static int failparse(void)
 {
     static const char *uris[] = {
-        "http://[::1/",
-        "http://[::1]f:80/",
-        "http://[::1]]:80/",
+	"http://[::1/",
+	"http://[::1]f:80/",
+	"http://[::1]]:80/",
         "http://foo/bar asda",
         "http://fish/[foo]/bar",
-        "http://foo:80bar",
-        "http://foo:80:80/bar",
-        NULL
+	NULL
     };
     int n;
     
@@ -528,8 +506,6 @@ static int resolve(void)
         ONN("bad pointer was returned", 
             ne_uri_resolve(&base, &relative, &resolved) != &resolved);
 
-        ONN("NULL path after resolve", resolved.path == NULL);
-
         actual = ne_uri_unparse(&resolved);
         
         ONCMP(ts[n].expected, actual, ts[n].relative, "output mismatch");
