@@ -1,6 +1,6 @@
 /* 
    neon-specific test utils
-   Copyright (C) 2001-2002, Joe Orton <joe@manyfish.co.uk>
+   Copyright (C) 2001-2005, Joe Orton <joe@manyfish.co.uk>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -23,6 +23,8 @@
 
 #include "ne_request.h"
 
+#include "child.h"
+
 #define ONREQ(x) do { int _ret = (x); if (_ret) { t_context("line %d: HTTP error:\n%s", __LINE__, ne_get_error(sess)); return FAIL; } } while (0);
 
 int single_serve_string(ne_socket *s, void *userdata);
@@ -54,4 +56,23 @@ int make_session(ne_session **sess, server_fn fn, void *userdata);
 /* Server which sleeps for 10 seconds then closes the socket. */
 int sleepy_server(ne_socket *sock, void *userdata);
 
+struct string {
+    char *data;
+    size_t len;
+};
+
+/* Serve a struct string. */
+int serve_sstring(ne_socket *sock, void *ud);
+
+/* Serve a struct string slowly. */
+int serve_sstring_slowly(ne_socket *sock, void *ud);
+
+struct infinite {
+    const char *header, *repeat;
+};
+
+/* Pass a "struct infinite *" as userdata, this function sends
+ * ->header and then loops sending ->repeat forever. */
+int serve_infinite(ne_socket *sock, void *ud);
+
 #endif /* UTILS_H */
